# -*- coding: utf-8 -*-
"""
/***************************************************************************
 FeatureStatics
                                 A QGIS plugin
 Feature's statistics information
                              -------------------
        begin                : 2016-04-07
        git sha              : $Format:%H$
        copyright            : (C) 2016 by Bob Zhou
        email                : enbozhou@pku.edu.cn
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
from PyQt4.QtCore import QSettings, QTranslator, qVersion, QCoreApplication
from PyQt4.QtGui import QAction, QIcon
# Initialize Qt resources from file resources.py
import resources
# Import the code for the dialog
from FeatureStatics_dialog import FeatureStaticsDialog
import os.path
from qgis.core import *
from qgis.gui import *


class FeatureStatics:
    """QGIS Plugin Implementation."""

    def __init__(self, iface):
        """Constructor.

        :param iface: An interface instance that will be passed to this class
            which provides the hook by which you can manipulate the QGIS
            application at run time.
        :type iface: QgsInterface
        """
        # Save reference to the QGIS interface
        self.iface = iface
        # initialize plugin directory
        self.plugin_dir = os.path.dirname(__file__)
        # initialize locale
        locale = QSettings().value('locale/userLocale')[0:2]
        locale_path = os.path.join(
            self.plugin_dir,
            'i18n',
            'FeatureStatics_{}.qm'.format(locale))

        if os.path.exists(locale_path):
            self.translator = QTranslator()
            self.translator.load(locale_path)

            if qVersion() > '4.3.3':
                QCoreApplication.installTranslator(self.translator)

        # Create the dialog (after translation) and keep reference
        self.dlg = FeatureStaticsDialog()

        # Declare instance attributes
        self.actions = []
        self.menu = self.tr(u'&FeatureStatics')
        # TODO: We are going to let the user set this up in a future iteration
        self.toolbar = self.iface.addToolBar(u'FeatureStatics')
        self.toolbar.setObjectName(u'FeatureStatics')

    # noinspection PyMethodMayBeStatic
    def tr(self, message):
        """Get the translation for a string using Qt translation API.

        We implement this ourselves since we do not inherit QObject.

        :param message: String for translation.
        :type message: str, QString

        :returns: Translated version of message.
        :rtype: QString
        """
        # noinspection PyTypeChecker,PyArgumentList,PyCallByClass
        return QCoreApplication.translate('FeatureStatics', message)


    def add_action(
        self,
        icon_path,
        text,
        callback,
        enabled_flag=True,
        add_to_menu=True,
        add_to_toolbar=True,
        status_tip=None,
        whats_this=None,
        parent=None):
        """Add a toolbar icon to the toolbar.

        :param icon_path: Path to the icon for this action. Can be a resource
            path (e.g. ':/plugins/foo/bar.png') or a normal file system path.
        :type icon_path: str

        :param text: Text that should be shown in menu items for this action.
        :type text: str

        :param callback: Function to be called when the action is triggered.
        :type callback: function

        :param enabled_flag: A flag indicating if the action should be enabled
            by default. Defaults to True.
        :type enabled_flag: bool

        :param add_to_menu: Flag indicating whether the action should also
            be added to the menu. Defaults to True.
        :type add_to_menu: bool

        :param add_to_toolbar: Flag indicating whether the action should also
            be added to the toolbar. Defaults to True.
        :type add_to_toolbar: bool

        :param status_tip: Optional text to show in a popup when mouse pointer
            hovers over the action.
        :type status_tip: str

        :param parent: Parent widget for the new action. Defaults None.
        :type parent: QWidget

        :param whats_this: Optional text to show in the status bar when the
            mouse pointer hovers over the action.

        :returns: The action that was created. Note that the action is also
            added to self.actions list.
        :rtype: QAction
        """

        icon = QIcon(icon_path)
        action = QAction(icon, text, parent)
        action.triggered.connect(callback)
        action.setEnabled(enabled_flag)

        if status_tip is not None:
            action.setStatusTip(status_tip)

        if whats_this is not None:
            action.setWhatsThis(whats_this)

        if add_to_toolbar:
            self.toolbar.addAction(action)

        if add_to_menu:
            self.iface.addPluginToVectorMenu(
                self.menu,
                action)

        self.actions.append(action)

        return action

    def initGui(self):
        """Create the menu entries and toolbar icons inside the QGIS GUI."""

        icon_path = ':/plugins/FeatureStatics/icon.png'
        self.add_action(
            icon_path,
            text=self.tr(u'FeatureStatistics'),
            callback=self.run,
            parent=self.iface.mainWindow())


    def unload(self):
        """Removes the plugin menu item and icon from QGIS GUI."""
        for action in self.actions:
            self.iface.removePluginVectorMenu(
                self.tr(u'&FeatureStatics'),
                action)
            self.iface.removeToolBarIcon(action)
        # remove the toolbar
        del self.toolbar


    def run(self):
        """Run method that performs all the real work"""
	layer = self.iface.activeLayer()#still need to verify the type of layer
	iter = layer.getFeatures()
	PointNum=0
	MultiPointNum=0
	LineStringNum=0
	LinearRingNum=0
	MultiLineStringNum=0
	PolygonNum=0
	MultiPolygonNum=0
	GeometryCollectionNum=0
	PointNumInLine=0
	PointNumInPolygon=0
	for feature in iter:
	    # retrieve every feature with its geometry and attributes
	    # fetch geometry
	    geom = feature.geometry()
	    # show some information about the feature
	    if geom.wkbType() == QGis.WKBPoint:
		PointNum=PointNum+1
	    elif geom.wkbType() == QGis.WKBLineString:
		LineStringNum=LineStringNum+1
		PointNumInLine=PointNumInLine+len(geom.asPolyline())
	    elif geom.wkbType() == QGis.WKBPolygon:
		PolygonNum=PolygonNum+1
		x = geom.asPolygon()
		numPts = 0
		for ring in x:
		    numPts += len(ring)
		PointNumInPolygon=PointNumInPolygon+numPts
	    elif geom.wkbType()==QGis.WKBMultiPoint:
		MultiPointNum=MultiPointNum+1
	    elif geom.wkbType()==QGis.WKBMultiLineString:
		MultiLineStringNum=MultiLineStringNum+1
	    elif geom.wkbType()==QGis.WKBMultiPolygon:
		MultiPolygonNum=MultiPolygonNum+1
	    elif geom.wkbType()==QGis.wkbLinearRing:
		LinearRing=LinearRing+1
	res="Total Geometry Number: %d \n" %(PointNum+MultiPointNum+LineStringNum+LinearRingNum+MultiLineStringNum+PolygonNum+MultiPolygonNum+GeometryCollectionNum)
	res=res+"Point Number: %d \n" % PointNum
	res=res+"MultiPoint Number: %d \n" % MultiPointNum
	res=res+"LineString Number: %d\n " % LineStringNum
	res=res+"LinearRing Number: %d \n" % LinearRingNum
	res=res+"MultiLineString Number: %d \n" % MultiLineStringNum
	res=res+"Polygon Number: %d \n" % PolygonNum
	res=res+"MultiPolygon Number: %d \n" % MultiPolygonNum
	res=res+"GeometryCollection Number: %d \n" % GeometryCollectionNum
	res=res+"Total Point Number in Line: %d \n" %PointNumInLine
	res=res+"Total Point Number in Polygon: %d \n" %PointNumInPolygon
	if LineStringNum==0:
	    res=res+"There is no Line Geometry!\n"
	else:
	    res=res+"Average Point Number in Line: %f \n" %(PointNumInLine*1.0/LineStringNum)
	if PolygonNum==0:
	    res=res+"There is no Polygon Geometry!\n"
	else:
	    res=res+"Average Point Number in Polygon: %f \n" %(PointNumInPolygon*1.0/PolygonNum)
	print res

        # show the dialog
    	self.dlg.show()
	self.dlg.showStatistics(res)    
        # Run the dialog event loop
    	result = self.dlg.exec_()
        # See if OK was pressed
     	if result:
	    canvas = QgsMapCanvas()
	    canvas.show()
            # Do something useful here - delete the line containing pass and
            # substitute with your code.
            pass

   
     
	#===========================================================================
	# print "Total Geometry Number: %d \n" %(PointNum+MultiPointNum+LineStringNum+LinearRingNum+MultiLineStringNum+PolygonNum+MultiPolygonNum+GeometryCollectionNum)
	# print "Point Number: %d \n" % PointNum
	# print "MultiPoint Number: %d \n" % MultiPointNum
	# print "LineString Number: %d\n " % LineStringNum
	# print "LinearRing Number: %d \n" % LinearRingNum
	# print "MultiLineString Number: %d \n" % MultiLineStringNum
	# print "Polygon Number: %d \n" % PolygonNum
	# print "MultiPolygon Number: %d \n" % MultiPolygonNum
	# print "GeometryCollection Number: %d \n" % GeometryCollectionNum
	# print "\n"
	# print "Total Point Number in Line: %d \n" %PointNumInLine
	# print "Total Point Number in Polygon: %d \n" %PointNumInPolygon
	# if LineStringNum==0:
	#     print "There is no Line Geometry!"
	# else:
	#     print "Average Point Number in Line: %f \n" %(PointNumInLine*1.0/LineStringNum)
	# if PolygonNum==0:
	#     print "There is no Polygon Geometry!"
	# else:
	#     print "Average Point Number in Polygon: %f \n" %(PointNumInPolygon*1.0/PolygonNum)
	#===========================================================================

     
